import System.Environment
-- Author: Stefan Witzel

-- wrds letters n
-- listet alle Worte der Länge n in letters auf
-- die Reihenfolge ist lexikographisch bezüglich der von letters induzierten Ordnung
wrds :: [ a ] -> Int -> [[ a ]]
wrds letters 0 = [[]]
wrds letters n = [ y:x | y <- letters, x <- (wrds letters (n - 1))]

-- wordCompare letters
-- gibt die von letters induzierte Ordnung zurück
wordCompare :: (Ord a) => [a] -> a -> a -> Ordering
wordCompare [] _ _  = error "Letters cannot be compared"
wordCompare (x:xs) y z
    | y == z    = EQ
    | x == y    = LT
    | x == z    = GT
    | otherwise = wordCompare xs y z

-- lexCompare cmp
-- gibt die zu cmp gehörende lexikographische Ordnung zurück
lexCompare :: (Ord a) => (a -> a -> Ordering) -> [ a ] -> [ a ] -> Ordering
lexCompare _   [] [] = EQ
lexCompare _   [] _ = LT
lexCompare _   _ [] = GT
lexCompare cmp (x:xs) (y:ys)
    | (x `cmp` y) == EQ = lexCompare cmp xs ys
    | otherwise = cmp x y

-- minRep cmp rep
-- gibt zurück ob rep <= (reverse rep) bezüglich cmp
minRep :: (Ord a) => ([ a ] -> [ a ] -> Ordering) -> [ a ] -> Bool
minRep cmp w = (w `cmp` (reverse w)) /= GT

-- bands letters n
-- listet die lexikographisch minimalen Bandrepräsentanten
-- in lexikographischer Reihenfolge auf
bands :: [ a ] -> Int -> [[ a ]]
bands letters n = [ x | x <- (wrds letters n), isRep x ]
    where isRep = minRep cmp
          cmp   = lexCompare wcmp
          wcmp  = wordCompare letters

main :: IO()
main = do
    args <- getArgs
    let letters:nstr:_ = args
    let n = read nstr :: Int
    print (bands letters n)
